/*
 * Es un simple watchDog que controla un tiempo y se pasa ese tiempo sin reiniciarlo llama a la funcion pasada
 */
var ControlTime = function() {
	function ControlTime( callback, time ) {
		this.callback = callback;
		this.time = time;
		this.timeout = null;
		this.start();
	}

	var _proto = ControlTime.prototype;

	_proto.start = function start() {
		var self = this;
		this.timeout = setTimeout( function() {
			if ( self.timeout ) {
				self.stop();
				self.callback();
			}
		}, this.time );
	};

	_proto.stop = function stop() {
		if ( this.timeout ) {
			clearTimeout( this.timeout );
		}

		this.timeout = null;
	};

	_proto.reset = function reset() {
		this.stop();
		this.start();
	};

	return ControlTime;
}();

/*
 * Controla el tiempo de sesión si se pasa abrirá el dialogo cerrando sesión. Si el control de eventos está activo,
 * controla el tiempo sin ajax si se excede ese tiempo se escucharán los eventos (por defecto keydown y mousedown) y
 * desde entonces cualquier evento lanzará petición ajax. Si llega una petición ajax se avisará a esta clase
 * (ajaxCalled) y se reinicia todo.
 */
var ControlTimers = function() {
	function ControlTimers( props ) {
		// console.log('-- ControlTimers: constructor');
		this.props = props;
		this.catchingEvents = false;
		this.start();
	}

	var _proto2 = ControlTimers.prototype;

	_proto2.start = function start() {
		var self = this;
		this.controlTimeSession = new ControlTime(
				function() { PF( self.props.dialogClosing ).show();	},
				this.props.sessionTime );
		if ( this.props.controlEvents ) {
			this.controlTimeWithoutAjax = new ControlTime(
				function() { self.catchEvents();},
				this.props.timeWithoutAjax );
		}
	};

	_proto2.callAjax = function callAjax() {
		// console.log('-- ControlTimers: callAjax');
		this.ajaxflag = false;
		var self = this;
		setTimeout( function() {
			if ( self.ajaxflag === false ) {
				refrescaSesion(); // Declarada en template.xhtml
			}
		}, 1000 );
	};

	_proto2.stop = function stop() {
		this.controlTimeSession.stop();
		if ( this.controlTimeWithoutAjax ) {
			this.controlTimeWithoutAjax.stop();
		}
		if ( this.catchingEvents ) {
			jQuery( document ).off( "._tempSession" );
			this.catchingEvents = false;
		}
	};

	_proto2.ajaxCalled = function ajaxCalled() {
		this.controlTimeSession.reset();
		this.ajaxflag = true;
		if ( this.controlTimeWithoutAjax ) {
			this.controlTimeWithoutAjax.reset();
		}
		if ( this.catchingEvents ) {
			jQuery( document ).off( "._tempSession" );
			this.catchingEvents = false;
		}
	};

	_proto2.catchEvents = function catchEvents() {
		// console.log("-- ControlTimers: capturando eventos " + this.props.events);
		this.catchingEvents = true;
		var self = this;
		jQuery( document ).on( jQuery.trim( ( this.props.events + " " ).split( " " ).join( "._tempSession " ) ),
	        function( e ) {
		        // console.log("-- ControlTimers: evento capturado");
		        jQuery( document ).off( "._tempSession" );
		        self.catchingEvents = false;
		        self.callAjax();
	        } );
	};

	return ControlTimers;
}();

/*
 * Si el control multiventana está activo se utilizará "localStorage" y se escucharán los eventos "storage" para
 * detectar cuando otra ventana ha lanzado petición ajax. Esta funcionalidad evita que una segunda ventana inactiva nos
 * cierre la sesión sin darnos cuenta.
 */
var ControlMultiWindow = function() {
	function ControlMultiWindow( props ) {
		// console.log('-- ControlMultiWindow: constructor');
		this.props = props; // Utilizamos window.location.hostname para que solo afecte a esta aplicacion
		this.localStorageKey = window.location.hostname + "-sessionController";
		this.start();
	}

	var _proto3 = ControlMultiWindow.prototype;

	_proto3.start = function start() {
		var self = this;
		jQuery( window ).on( "storage", function( e ) {
			if ( e.type === "storage" && e.originalEvent.key === self.localStorageKey ) {
				console.log( 'sessionController storage: ' + e.originalEvent.newValue );
				self.ajaxExterno( e.originalEvent.newValue );
			}
		} );
	};

	_proto3.ajaxExterno = function ajaxExterno( value ) {
		if ( value === "0" ) {
			// Indica que otra ventana ha finalizado sesion
			sessionController.finalizedSession();
			PF( this.props.dialogClosing ).hide();
			PF( this.props.dialogNoSession ).show();
		}
		if ( !sessionController.finalized ) {
			if ( sessionController.cTimers ) {
				sessionController.cTimers.ajaxCalled();
			}
			sessionController.continuedSession();
		}
	};

	_proto3.ajaxCalled = function ajaxCalled() {
		this.guardaValorLocalStorage( +new Date() );
	};

	_proto3.endSession = function endSession() {
		this.guardaValorLocalStorage( 0 ); // Indicamos que ha finalizado la sesion
		jQuery( window ).off( "storage" );
	};

	_proto3.guardaValorLocalStorage = function guardaValorLocalStorage( valor ) {
		if ( localStorage ) {
			// console.log('-- guardaFechaUltimoAjax en localStorage: ' + valor);
			localStorage.setItem( this.localStorageKey, valor );
		}
	};

	return ControlMultiWindow;
}();

/*
 * Es el objeto principal, con la que se interactua, usa las clases ControlMultiWindow y ControlTimers, les pasa la
 * información necesaria y también controla el dialogo cerrando y su contador.
 */
var sessionController = {
    props : {
        // Propiedades por defecto
        dialogClosing : 'cerrandoSesionDlg',
        dialogNoSession : 'noSesionDlg',
        sessionTime : 1500000,
        timeWithoutAjax : 600000,
        events : 'keydown mousedown',
        minuteText : 'm',
        minutesText : 'm',
        secondText : 's',
        secondsText : 's',
        controlEvents : true,
        multiWindow : true,
        dialogSeconds : 120
    },
    finalized : false,
    // Llamado desde fdw:onloadScript
    ajaxCalled : function ajaxCalled( props ) {
	    if ( !this.finalized ) {
		    // console.log('sessionController ajaxLlamado');
		    if ( !this.cTimers ) {
			    // Es la primera vez que se llama
			    this.start( props );
		    }
		    else {
			    this.cTimers.ajaxCalled();
		    }
		    if ( this.mWindow ) {
			    this.mWindow.ajaxCalled();
		    }
	    }
    },
    setProps : function setProps( props ) {
	    jQuery.extend( this.props, props );
    },
    start : function start( props ) {
	    console.log( 'sessionController start' );
	    if ( props ) {
		    jQuery.extend( this.props, props );
	    }
	    if ( !jQuery.isNumeric( this.props.sessionTime ) ) {
		    this.props.sessionTime = 1500000; // 25 minutos
		    console.log( 'sessionTime error, default ' + this.props.sessionTime );
	    }
	    if ( !jQuery.isNumeric( this.props.timeWithoutAjax ) ) {
		    this.props.timeWithoutAjax = 600000; // 10 minutos
		    console.log( 'timeWithoutAjax error, default ' + this.props.timeWithoutAjax );
	    }

	    this.cTimers = new ControlTimers( this.props );
	    if ( this.props.multiWindow ) {
		    this.mWindow = new ControlMultiWindow( this.props );
	    }
    },
    // Llamado cuando se muestra el dialogo cerrando
    showingClosingDialog : function showingClosingDialog( btnInvalidateId, counterId ) {
	    var self = this;
	    var toMinSec = function toMinSec( secs ) {
		    var timeStr = '';
		    var minutes = Math.floor( secs / 60 );
		    if ( minutes > 0 ) {
			    timeStr += minutes + ' ' + ( minutes > 1 ? self.props.minutesText : self.props.minuteText ) + ' ';
		    }
		    var seconds = secs % 60;
		    if ( seconds > 0 ) {
			    timeStr += seconds + ' ' + ( seconds > 1 ? self.props.secondsText : self.props.secondText ) + ' ';
		    }
		    else if ( secs === 0 ) {
			    timeStr += '0 ' + self.props.secondsText;
		    }
		    return timeStr;
	    };

	    var time = this.props.dialogSeconds;
	    if ( this.cTimers ) {
		    this.cTimers.stop();
	    }
	    var ele = document.getElementById( counterId );
	    var btn = document.getElementById( btnInvalidateId );
	    if ( this.waitCounter ) {
		    clearInterval( this.waitCounter );
	    }

	    ele.innerHTML = toMinSec( time );
	    this.waitCounter = setInterval( function() {
		    if ( time === 0 ) {
			    clearInterval( self.waitCounter );
			    self.waitCounter = null;
			    btn.click();
		    }
		    else {
			    time--;
			    ele.innerHTML = toMinSec( time );
		    }
	    }, 1000 );
    },
    // Llamado cuando se ha pulsado el botón continua sesion
    continuedSession : function continuedSession() {
	    // console.log('sessionController continuedSession');
	    if ( this.waitCounter ) {
		    clearInterval( this.waitCounter );
		    this.waitCounter = null;
		    PF( this.props.dialogClosing ).hide();
	    }
    },
    // Llamado cuando se ha pulsado el botón cierra sesion
    finalizedSession : function finalizedSession() {
	    // console.log('sessionController finalizedSession');
	    if ( this.waitCounter ) {
		    clearInterval( this.waitCounter );
		    this.waitCounter = null;
	    }
	    if ( this.cTimers ) {
		    this.cTimers.stop();
	    }
	    if ( this.mWindow ) {
		    this.mWindow.endSession();
	    }
	    this.finalized = true;
    }
};
